def get_smaller_value(val1, val2):
    """
    Function compares two values and returns smaller one.

    Parameters
    ----------
    val1 : number

    val2 : number

    Returns
    -------
    : number
    """
    if val1 < val2:
        return val1
    return val2


def get_larger_value(val1, val2):
    """
    Function compares two values and returns larger one.

    Parameters
    ----------
    val1 : number

    val2 : number

    Returns
    -------
    : number
    """
    if val1 > val2:
        return val1
    return val2
